#!/usr/bin/env python
# -*- coding: utf-8 -*-
 
"""
    Module pyant
    ant by python
"""
import os, sys
import os.path
import glob
import re

from mytools.util import *

__version__ = '0.0.4'


GET_BOOLEAN = {'true' : True,
                'false' : False,
                0 : False,
                1 : True,
                True : True,
                False : False}

LOG = 'classic'

def import_global(mod):
   globals()[mod] = __import__(mod) 

def subant(g):
   return '%(' + g.group()[2:-1] + ')s'


class Elt(object):

    ATTR_REQUIRED = []
    ATTR_DEFAULT = {}
    ATTR_POSSIBLE_VALUE = {}

    def __init__(self, **kwargs):
        self._poslog = 0
        self.parent = None
        self._append_obj = {}
        self._kwargs = ['_%s' % i for i in kwargs.keys()]
        for i in kwargs:
            setattr(self, i, kwargs[i])
        for i in self.ATTR_DEFAULT:
            if i not in kwargs:
                setattr(self, i, self.ATTR_DEFAULT[i])
                self._kwargs.append('_%s' % i)

    def _checkadd(self):
        pass

    def _check(self):
        for i in self.ATTR_REQUIRED:
            if i not in dir(self):
                raise AttributeError("%s does not attribute %s" % (self.__class__.__name__, i))
        for i in self.ATTR_POSSIBLE_VALUE:
            if i in dir(self) and self.__getattribute__('_%s' % i) not in self.ATTR_POSSIBLE_VALUE[i]:
                raise AttributeError("%s not possible value of %s" % (i, self.__class__.__name__))
        self._checkadd()

    def _propertys(self):
        a = Propertys()
        for i in self._append_obj.get('Property',[]):
            if i.name:
                a[i.name] = i.value
        if self.parent:
            a += self.parent._propertys()   
        return a    

    def append_obj(self, obj):
        if isinstance(obj, Property) and obj.__class__.__name__  in self._append_obj :
            for i in self._append_obj[obj.__class__.__name__] :
                if i.name == obj.name:
                    self._append_obj[obj.__class__.__name__].remove(i)
        self._append_obj[obj.__class__.__name__] = [obj,] if obj.__class__.__name__ not in self._append_obj else self._append_obj[obj.__class__.__name__] + [obj,]
        obj.parent = self

    def __getattribute__(self, item):
        if item != '_kwargs' and item in self._kwargs :
            try:
                return re.sub(r'\${[a-zA-Z0-9.]*}',subant, self.__getattribute__(item[1:])) % self._propertys()
            except:
                return self.__getattribute__(item[1:]) #if problem of %
        if item == 'project':
            if self.parent:
                if self.parent.__class__.__name__ == "Project":
                    return self.parent
                return self.parent.project    
            else:
                raise AttributeError("project not found")
        if item == 'mytarget':
            if self.parent:
                if self.parent.__class__.__name__ == "Target":
                    return self.parent
                return self.parent.mytarget    
            else:
                raise AttributeError("target not found")
        return object.__getattribute__(self, item)
        #manage KeyError? generate PropertyError                             

    def _end_init(self):
        self._check()
        for k in self._append_obj:
            for obj in self._append_obj[k]:
                if isinstance(obj,Elt):
                    obj._end_init()

    def log(self, level, msg):
        if LOG == 'simple':
            Logger().log(level, '%s' % (msg))
            return
        Logger().log(level, '%s[%s] %s' % ('\t' * self._poslog, self.__class__.__name__, msg))

    def copy(self):
        kwargs = dict([[i[1:], self.__getattribute__(i[1:])] for i in self._kwargs])
        n =  self.__class__(**kwargs)
        for i in self._append_obj.keys():
            for j in  self._append_obj[i]:
                n.append_obj(j.copy())
        n.parent = self
        n._end_init()
        return n

class Property(Elt):
    """
    Sets a property (by name and value). Properties are case sensitive.
    Properties are immutable: whoever sets a property first freezes it for the rest of the build;
    they are most definitely not variables. 
    
    name of element : **property**

    ==================== ============================================= ========
    Attribute            Information                                   Required
    ==================== ============================================= ========
    name                 name of property                              Yes
    value                default ''
    eval                 eval value, default false
    global               add a property a project level default false
    ==================== ============================================= ========
    """
 
    ATTR_REQUIRED = ['name']
    ATTR_DEFAULT = {'name' : None, 
            'value' : '',
            'eval' : 'false',
            'global' : 'false'}
    ATTR_POSSIBLE_VALUE = { 'eval' : ['true','false'],
                            'global' : ['true','false']} 


    def __init__(self, **kwargs):
        Elt.__init__(self, **kwargs)

    def _end_init(self):
        #manage attr global
        if self._global == 'true':
            new_parent = self.project
            self.parent._append_obj[self.__class__.__name__].remove(self)
            new_parent.append_obj(self)     
        Elt._end_init(self)

    def __getattr__(self, item):
        if item == 'value' and 'location' in dir(self):
            return self.location
        raise AttributeError(item)

    def __setattr__(self, item, value):
        if item == 'cdata':
            item = 'value'
        Elt.__setattr__(self, item, value)

    def __getattribute__(self, item):
        if item == 'value' and self.eval == 'true':
            self.eval = 'false'
            val = Elt.__getattribute__(self, item)
            self.eval = 'true'
            try:
                return eval(val)
            except:
                return ''
        return Elt.__getattribute__(self, item)
 


class Propertys(dict):

    def __init__(self):
        super(Propertys, self).__init__()
    
    def __add__(self,y):
        if y.__class__.__name__ == self.__class__.__name__:
            for i in y:
                if i not in self:
                    self[i] = y[i]
            return self
        else:
            raise TypeError('unsupported operand type(s) for %s and %s' % (self.__class__.__name__, y.__class__.__name__))
    
    def __iadd__(self, y):
        return self.__add__(y)


class Param(Elt):
    """
    Add parameter for other element
    
    name of element : **param**

    ==================== ============================================= ========
    Attribute            Information                                   Required
    ==================== ============================================= ========
    name                 name of property                              Yes
    value                default ''
    type                 str, int, bool, float, eval default str
    ==================== ============================================= ========
    """
 
    ATTR_REQUIRED = ['name']
    ATTR_DEFAULT = {'value' : '',
                    'type' : 'str'}
    ATTR_POSSIBLE_VALUE = {'type' : ['str','int', 'bool', 'float', 'eval'] } 


    def __init__(self, **kwargs):
        Elt.__init__(self, **kwargs)

    def __setattr__(self, item, value):
        if item == 'cdata':
            item = 'value'
        Elt.__setattr__(self, item, value)

    def get(self):
        if self._type == 'int':
            return int(self._value)
        if self._type == 'float':
            return float(self._value)
        if self._type == 'bool':
            return bool(self._value)
        if self._type == 'eval':
            try:
                return eval(self._value)
            except:
                pass
        return self._value    

class Task(Elt):

    def __init__(self, **kwargs):
        Elt.__init__(self, **kwargs)

    def __getattribute__(self, item):
        if item in ('error', '_error') and '_error' not in self._kwargs :
            return None
        return Elt.__getattribute__(self, item) 

    def _run(self):
        pass

    def run(self, action, pos=0):
        self._poslog = pos
        if action:
            try:
                self._run()
            except Exception as e:
                if self._error  and e.__class__.__name__ != 'AntStopException':
                    self.log('DEBUG','manage error')
                    self.project.append_obj(Property(name='ant.error', value=str(e)))
                    self.project.get_target(self._error).run(action, pos)
                else:
                    raise e
        else:
            self.log('INFO', '')

class Target(Elt):
    """
    A target is a container of tasks that cooperate to reach a desired state
    during the build process. 
    
    name of element : **target**

    ==================== ============================================= ========
    Attribute            Information                                   Required
    ==================== ============================================= ========
    name                 name of target                                 Yes
    depends              list of target load before
    if                   load target if true, default true
    unless               load target if false default false
    description          
    error                target to load if error (default '')
    ==================== ============================================= ========
    
    All tasks with an error management with attribute error (call target when error) 
    
    You can use the property ant.error to know the value
    """
 
    ATTR_REQUIRED = ['name']
    ATTR_DEFAULT = { 'depends' : '',
                        'if' : 'true',
                        'unless' : 'false',
                        'description' : 'no description',
                        'error' : ''}

    def __init__(self, **kwargs):
        Elt.__init__(self, **kwargs)
        self.mydepends = self.depends.split(',') if len(self.depends) else []
        self._task = []

    def append_obj(self, obj):
        if isinstance(obj, Task):
            self._task.append(obj)
        Elt.append_obj(self, obj)

    def run(self, action=True, pos=0):
        self._poslog = pos
        self.log('INFO', self.name)
        if GET_BOOLEAN.get(self._if,False) and not GET_BOOLEAN.get(self._unless, True): 
            for t in self.mydepends:
                try:
                    self.project.get_target(t).run(action, pos+1)
                except Exception as e:
                    if self._error and e.__class__.__name__ != 'AntStopException':
                        self.log('DEBUG','%s manage error target %s' % (self.name, t))
                        self.project.append_obj(Property(name='ant.error', value=str(e)))
                        self.project.get_target(self._error).run(action, pos)
                    else:
                        raise e
            for t in self._task:
                try:
                    t.run(action, pos+1)
                except Exception as e:
                    if self._error and e.__class__.__name__ != 'AntStopException':
                        self.log('DEBUG','%s manage error task %s' % (self.name, t.__class__.__name__))
                        self.project.append_obj(Property(name='ant.error', value=str(e)))
                        self.project.get_target(self._error).run(action, pos)
                    else:
                        raise e
        else:
            self.log('INFO',"Skipped because property if is 'false' or unless is 'true'." ) 

class Project(Elt):
    """
    A project is a container of target
    
    name of element : **project**

    ==================== ============================================= ========
    Attribute            Information                                   Required
    ==================== ============================================= ========
    default              target default                                Yes
    basedir              getcwd default '.'
    name                 name of project
    ==================== ============================================= ========
    """
 
    ATTR_REQUIRED = ['default']
    ATTR_DEFAULT = { 'basedir' : '.',
                        'name' : 'No Name'}#,
                        #'file' : ''}

    def __init__(self, **kwargs):
        Elt.__init__(self, **kwargs)
        self._target = {}

    def append_obj(self, obj):
        if isinstance(obj, Target):
            self._target[obj.name] = obj
        Elt.append_obj(self, obj)

    def get_target(self, name, default= None):
        if default == None:
            try:
                return self._target[name]
            except:
                raise NameError("target %s not found" % name)
        return self._target.get(name,default)

    @property
    def targets(self):
        return self._target.values()

    def _add_propertys(self):
        self.append_obj(Property(name='ant.version', value=__version__))
        self.append_obj(Property(name='ant.project.name', value=self.name))
        self.append_obj(Property(name='ant.project.default-target', value=self.default))
        self.append_obj(Property(name='ant.python.version', value=sys.version.split(' ')[0])) 
        self.append_obj(Property(name='basedir', value=self.basedir))


    def run(self, t, action=True):
        os.chdir(self.basedir)
        self._add_propertys() 
        self._end_init() 
        for i in t:
            self.get_target(i).run(action)


ANTCLASS = {
            'project': Project,
            'target' : Target,
            'mkdir' :  Task,
            'property' : Property,
            'param' : Param,
           }

# import extensions
for i in glob.glob(os.path.join(os.path.dirname(__file__), 'extensions', '*.py')):
    if os.path.isfile(i):
        modname = 'pyant.extensions.%s' % os.path.split(i)[1][:-3]
        try:
            __import__(modname)
        except ImportError as e:
            raise SystemExit('\nError: Could not import extension %s'
                                % os.path.split(i)[1][:-3])


# /!\ ajouter tempfile
# /!\ voir pour faire un for avec un eval?
# /!\ voir ftp :http://effbot.org/librarybook/ftplib.htm 
