#!/usr/bin/env python
# -*- coding: utf-8 -*-
 
"""
    Module pyant
    ant by python
"""
import os, sys
import os.path
import pyant
try:
    import ConfigParser as configparser
except:
    import configparser
try:
    from tkinter import Tk
    from tkinter.messagebox import showerror, showinfo, showwarning
except:
    from Tkinter import Tk
    from tkMessageBox import showerror, showinfo, showwarning


from mytools.util import *

class Echo(pyant.Task):
    """
    Echoes a message to the current loggers
    
    name of element : **echo**

    ==================== ============================================= ========
    Attribute            Information                                   Required
    ==================== ============================================= ========
    message              message print (or cdata)
    level                default note (note, error, warning, info, 
                         verbose, debug)
    msgbox               view message in msgbox (default: false)
    file                 output message in file
    append               append to an existing file (default: false)
    ==================== ============================================= ========
    """
    

    LEVEL = {'note' : NOTE,
            'error' : CRITICAL,
            'warning' : WARNING,
            'info' : INFO,
            'verbose' : INFO,
            'debug' : DEBUG}

    LEVELMSGBOX = {'note' : showinfo,
            'error' : showerror,
            'warning' : showwarning,
            'info' : showinfo,
            'verbose' : showinfo,
            'debug' : showinfo}



    MODE = {'false' : 'w',
            'true' : 'a'}

    ATTR_REQUIRED = []
    ATTR_DEFAULT = {'message' : '',
                    'level' : 'note',
                    'append': 'false',
                    'msgbox': 'false'}
    ATTR_POSSIBLE_VALUE = {'level' : LEVEL.keys(),
                            'msgbox' : ['true','false'],
                            'append' : MODE.keys()} 

    def __init__(self, **kwargs):
        pyant.Task.__init__(self, **kwargs)

    def __setattr__(self, item, value):
        if item == 'cdata':
            item = 'message'
        pyant.Task.__setattr__(self, item, value)

    def _run(self):
        self.log(self.LEVEL[self._level], self._message)
        if self._msgbox == 'true':
            Tk_Appli = Tk()
            Tk_Appli.withdraw()
            self.LEVELMSGBOX[self._level](self._level, self._message)
            Tk_Appli.destroy()
        if 'file' in dir(self):
            if not os.path.exists(os.path.dirname(os.path.join(self.project.basedir, self._file))):
               os.makedirs(os.path.dirname(os.path.join(self.project.basedir, self._file))) 
               self.log('DEBUG','Created dir : %s' % os.path.dirname(os.path.join(self.project.basedir, self._file)))     
            with open(os.path.join(self.project.basedir, self._file), self.MODE[self._append]) as f:
                f.write(self._message)

pyant.ANTCLASS['echo'] = Echo


class AntException(Exception):

    def __init__(self, msg):
        self.msg = msg

    def __str__(self):
        return self.msg

class AntStopException(Exception):

    def __init__(self, msg):
        self.msg = msg

    def __str__(self):
        return self.msg


class Raise(pyant.Task):
    """
    Raise generate exception
    
    name of element : **raise**

    ==================== ============================================= ========
    Attribute            Information                                   Required
    ==================== ============================================= ========
    message              message of exception
    ==================== ============================================= ========
    """
    
    ATTR_REQUIRED = []
    ATTR_DEFAULT = {'message' : 'raise exception'}

    def __init__(self, **kwargs):
        pyant.Task.__init__(self, **kwargs)

    def __setattr__(self, item, value):
        if item == 'cdata':
            item = 'message'
        pyant.Task.__setattr__(self, item, value)

    def _run(self):
        self.log('DEBUG','generate Ant exception')
        raise AntException(self._message)

pyant.ANTCLASS['raise'] = Raise


class Fail(pyant.Task):
    """
    fail stop ant
    
    name of element : **fail**

    ==================== ============================================= ========
    Attribute            Information                                   Required
    ==================== ============================================= ========
    message              message of exception
    ==================== ============================================= ========
    """
    
    ATTR_REQUIRED = []
    ATTR_DEFAULT = {'message' : 'fail exception'}

    def __init__(self, **kwargs):
        pyant.Task.__init__(self, **kwargs)

    def __setattr__(self, item, value):
        if item == 'cdata':
            item = 'message'
        pyant.Task.__setattr__(self, item, value)

    def _run(self):
        self.log('DEBUG','generate Ant exception')
        raise AntStopException(self._message)

pyant.ANTCLASS['fail'] = Fail

class Available(pyant.Task):
    """
    Sets a property if a resource is available at runtime
    
    name of element : **available**

    ==================== ============================================= ========
    Attribute            Information                                   Required
    ==================== ============================================= ========
    property             the name of property                          Yes
    value                the value set the property to. Default true
    notvalue             the value not set the property. Default false 
    file                 the file to look for                          Yes
    type                 type of file (dir,file or all). Default all
    ==================== ============================================= ========
    """
    
    ATTR_REQUIRED = ['property','file']
    ATTR_DEFAULT = {'type' : 'all',
                    'value' : 'true',
                    'notvalue' :'false'}
    ATTR_POSSIBLE_VALUE = {'type' : ['dir','file','all'] } 

    def __init__(self, **kwargs):
        pyant.Task.__init__(self, **kwargs)

    def _add_property(self, val):
        self.parent.append_obj(pyant.Property(name=self._property, value=val))

    def _run(self):
        f = os.path.join(self.project.basedir, self._file)  
        if (self._type == 'all' and os.path.exists(f)) or (self._type == 'file' and os.path.isfile(f)) or (self._type == 'dir' and os.path.isdir(f)):
            self._add_property(self._value)
        else:
            self._add_property(self._notvalue)

pyant.ANTCLASS['available'] = Available

class Antcall(pyant.Task):
    """
    Call another target within the same buildfile
    
    name of element : **antcall**

    ==================== ============================================= ========
    Attribute            Information                                   Required
    ==================== ============================================= ========
    target               the target to execute                         Yes
    inheritAll           pass all properties .Default true
    ==================== ============================================= ========
    """
    
    ATTR_REQUIRED = ['target']
    ATTR_DEFAULT = {'inheritAll' : 'true'}
    ATTR_POSSIBLE_VALUE = {'inheritAll' : ['true','false'] } 

    def __init__(self, **kwargs):
        pyant.Task.__init__(self, **kwargs)

    def _add_property(self, val):
        self.parent.append_obj(pyant.Property(name=self._property, value=val))

    def _run(self):
        t = self.project.get_target(self._target)
        if self._inheritAll == 'true':
            t = t.copy()
            t.parent = self
        t.run(True, self._poslog+1)
 
pyant.ANTCLASS['antcall'] = Antcall


class Loadproperties(pyant.Task):
    """
    load a file's content properties
    
    name of element : **loadproperties**

    ==================== ============================================= ========
    Attribute            Information                                   Required
    ==================== ============================================= ========
    srcFile              source file                                   Yes
    prefix               prefix to apply to loaded properties
    ==================== ============================================= ========
    """
    
    ATTR_REQUIRED = ['srcFile']
    ATTR_DEFAULT = {'prefix' : ''}
    ATTR_POSSIBLE_VALUE = {} 

    def __init__(self, **kwargs):
        pyant.Task.__init__(self, **kwargs)

    def _add_property(self, prop, val):
        if len(self._prefix):
            prop = "%s.%s" % (self._prefix, prop)
        self.parent.append_obj(pyant.Property(name=prop, value=val))
    
    def _run(self):
        f = os.path.join(self.project.basedir, self._srcFile)   
        try:
            config = configparser.ConfigParser()
            config.read(f)
            for sec in config.sections():
                for opt in config.options(sec):
                    self._add_property("%s.%s" % (sec, opt), config.get(sec,opt))
        except Exception as e:
            with open(f, 'r') as config:
                for line in config.readlines():
                    self._add_property(line.split('=')[0], line.split('=')[1])

pyant.ANTCLASS['loadproperties'] = Loadproperties

class EnvironTask(pyant.Task):
    """
    append path to environ
    
    name of element : **environ**

    ==================== ============================================= ========
    Attribute            Information                                   Required
    ==================== ============================================= ========
    directory            directory to modified                         Yes
    path                 path to append in directory default .
    ==================== ============================================= ========
    """
    
    ATTR_REQUIRED = ['directory']
    ATTR_DEFAULT = {'path' : '.'}
    ATTR_POSSIBLE_VALUE = {} 

    def __init__(self, **kwargs):
        pyant.Task.__init__(self, **kwargs)
    
    def _run(self):
        p = os.path.join(self.project.basedir, self._path) 
        os.environ[self._directory] += os.pathsep + p

pyant.ANTCLASS['environ'] = EnvironTask


class IfTask(pyant.Task):
    """
    test condition and load tasks son
    
    name of element : **if**

    ==================== ============================================= ========
    Attribute            Information                                   Required
    ==================== ============================================= ========
    condition            default true
    ==================== ============================================= ========
    
    it's not authorized that use if in if
    """
    
    ATTR_REQUIRED = []
    ATTR_DEFAULT = {'condition' : 'true'}

    def __init__(self, **kwargs):
        pyant.Task.__init__(self, **kwargs)
        self._task = []

    def append_obj(self, obj):
        if isinstance(obj, pyant.Task):
            self._task.append(obj)
        pyant.Task.append_obj(self, obj)

    def _run(self):
        self.log('DEBUG','t if, value: %s' % self._condition)
        if pyant.GET_BOOLEAN.get(self._condition,True):
            for t in self._task:
                try:
                    t.run(True, self.parent._poslog+1)
                except Exception as e:
                    if self.parent._error and e.__class__.__name__ != 'AntStopException':
                        self.log('DEBUG','%s manage error task %s' % (self.name, t.__class__.__name__))
                        self.project.get_target(self.parent._error).run(action, pos)
                    else:
                        raise e
        else:
            self.log('INFO',"Skipped because condition is 'false'" ) 

  

pyant.ANTCLASS['if'] = IfTask



