#!/usr/bin/env python
# -*- coding: utf-8 -*-
 
"""
    Module pyant
    ant by python
"""
import os, sys
import os.path
import pyant
from mytools.util import *

try:
    #python 2.7
    from urllib import urlencode, urlopen
    from urllib2 import HTTPPasswordMgrWithDefaultRealm ,HTTPBasicAuthHandler , build_opener, install_opener
except:
    #python 3
    from urllib.parse import urlencode
    from urllib.request import urlopen,HTTPPasswordMgrWithDefaultRealm,HTTPBasicAuthHandler , build_opener, install_opener

class GetTask(pyant.Task):
    """
    get file from url
     
    name of element : **get**
    
    ==================== ============================================= ========
    Attribute            Information                                   Required
    ==================== ============================================= ========
    src                  the url                                       Yes
    dest                 file store the retrieved file                 Yes
    server               server for basic authentification
    username             user for basic authentification
    password             password for basic authentification
    failonerror          stop if failure default true 
    ==================== ============================================= ========
    
    You can use element param for add parameter of url
    """
   
    ATTR_REQUIRED = ['src','dest']
    ATTR_DEFAULT = {'failonerror' : 'true',
                    'server' : '',
                    'username' : '',
                    'password' : ''}
    ATTR_POSSIBLE_VALUE = {'failonerror' : ['true','false'] } 

    def __init__(self, **kwargs):
        pyant.Task.__init__(self, **kwargs)
        self.param = {}

    def _checkadd(self):
       if 'Param' in self._append_obj.keys():
           for i in self._append_obj['Param']:
               self.param[i.name] = i.get() 

    def _run(self):
        try:
            if len(self._server):
                authinfo = HTTPPasswordMgrWithDefaultRealm() 
                authinfo.add_password(None, self._server, self._username, self._password)
                handler = HTTPBasicAuthHandler(authinfo)
                myopener = build_opener(handler)
                opened = install_opener(myopener)
            a = urlopen(self._src + urlencode(self.param))
            with open(os.path.join(self.project.basedir, self._dest), 'w') as f:
                f.write(a.read())
            self.log('INFO','get on %s' % self._src)
        except Exception as e:
            if self._failonerror == 'false':
                self.log('DEBUG','error on get')
            else:
                raise e

pyant.ANTCLASS['get'] = GetTask

