#!/usr/bin/env python
# -*- coding: utf-8 -*-
 
"""
    Module pyant
    ant by python
"""
import os, sys
import os.path
import pyant
import copy
from mytools.util import *

import smtplib
from email.mime.multipart import MIMEMultipart
from email.mime.text import MIMEText
from email.MIMEBase import MIMEBase
from email.MIMEImage import MIMEImage
from email.Utils import COMMASPACE, formatdate
from email import Encoders


class From(pyant.Elt):
    """
    use with element **mail**
     
    name of element : **from**
    
    ==================== ============================================= ========
    Attribute            Information                                   Required
    ==================== ============================================= ========
    address              email address
    ==================== ============================================= ========
    """
 
    ATTR_REQUIRED = []
    ATTR_DEFAULT = {'address' : '' }

    def __init__(self, **kwargs):
        pyant.Elt.__init__(self, **kwargs)

    def __setattr__(self, item, value):
        if item == 'cdata':
            item = 'address'
        pyant.Elt.__setattr__(self, item, value)

pyant.ANTCLASS['from'] = From

class To(From):
    """
    use with element **mail**
     
    name of element : **to**
    
    ==================== ============================================= ========
    Attribute            Information                                   Required
    ==================== ============================================= ========
    address              email address
    ==================== ============================================= ========
    """
 
    ATTR_REQUIRED = []
    ATTR_DEFAULT = {'address' : '' }

    def __init__(self, **kwargs):
        From.__init__(self, **kwargs)

pyant.ANTCLASS['to'] = To

class Cc(From):
    """
    use with element **mail**
     
    name of element : **cc**
    
    ==================== ============================================= ========
    Attribute            Information                                   Required
    ==================== ============================================= ========
    address              email address
    ==================== ============================================= ========
    """
 
    ATTR_REQUIRED = []
    ATTR_DEFAULT = {'address' : '' }

    def __init__(self, **kwargs):
        From.__init__(self, **kwargs)

pyant.ANTCLASS['cc'] = Cc

class Bcc(From):
    """
    use with element **mail**
     
    name of element : **bcc**
    
    ==================== ============================================= ========
    Attribute            Information                                   Required
    ==================== ============================================= ========
    address              email address
    ==================== ============================================= ========
    """
 
    ATTR_REQUIRED = []
    ATTR_DEFAULT = {'address' : '' }

    def __init__(self, **kwargs):
        From.__init__(self, **kwargs)

pyant.ANTCLASS['ccc'] = Bcc

class Replyto(From):
    """
    use with element **mail**
     
    name of element : **replyto**
    
    ==================== ============================================= ========
    Attribute            Information                                   Required
    ==================== ============================================= ========
    address              email address
    ==================== ============================================= ========
    """
 
    ATTR_REQUIRED = []
    ATTR_DEFAULT = {'address' : '' }

    def __init__(self, **kwargs):
        From.__init__(self, **kwargs)

pyant.ANTCLASS['replyto'] = Replyto

class Message(pyant.Elt):
    """
    use with element **mail**
     
    name of element : **message**
    
    ==================== ============================================= ========
    Attribute            Information                                   Required
    ==================== ============================================= ========
    value                or cdata
    ==================== ============================================= ========
    """
 
    ATTR_REQUIRED = []
    ATTR_DEFAULT = {'value' : '' }

    def __init__(self, **kwargs):
        pyant.Elt.__init__(self, **kwargs)

    def __setattr__(self, item, value):
        if item == 'cdata':
            item = 'value'
        pyant.Elt.__setattr__(self, item, value)


pyant.ANTCLASS['message'] = Message

class Attachments(pyant.Elt):
    """
    use with element **mail**
     
    name of element : **attachments**
    
    ==================== ============================================= ========
    Attribute            Information                                   Required
    ==================== ============================================= ========
    address              email address
    ==================== ============================================= ========
    """
 
    ATTR_REQUIRED = []
    ATTR_DEFAULT = {'value' : '' }

    def __init__(self, **kwargs):
        pyant.Elt.__init__(self, **kwargs)

    def __setattr__(self, item, value):
        if item == 'cdata':
            item = 'value'
        pyant.Elt.__setattr__(self, item, value)


pyant.ANTCLASS['attachments'] = Attachments


class Mail(pyant.Task):
    """
    to send SMTP email
     
    name of element : **mail**
    
    ==================== ============================================= ========
    Attribute            Information                                   Required
    ==================== ============================================= ========
    from                 Email address of sender                       Yes
    replyto              repltyto email address
    tolist               list of recipient                             Yes
    cclist               list of recipient to carbon copy
    bcclist              list of recipient to blind carbon copy
    message              body of email
    messagefile          file to send as the body of the mail
    messagemimetype      default plain (possible html)
    files                files to send as attachments to the email
    failonerror          stop if failure default true 
    mailhost             host name of SMTP server default localhost
    mailport             TCP port of the SMTP server default 25
    user                 user name for SMTP auth
    password             password for SMTP auth
    ssl                  default false
    subject              email subject fline default "pyant email"
    ==================== ============================================= ========
    
    You can use element to, cc, bcc, replyto, message, 
    attachments instead of attributes
    """
   
    ATTR_REQUIRED = ['from']
    ATTR_DEFAULT = {'replyto' : '',
                    'cclist' : '',
                    'bcclist' : '',
                    'message' : '',
                    'messagefile' : '',
                    'messagemimetype' : 'plain',
                    'files' : '',
                    'failonerror' : 'true',
                    'mailhost' : 'locahost',
                    'mailport' : '25',
                    'user' : '',
                    'password' : '',
                    'ssl' : 'false',
                    'subject' : 'pyant email'}
    ATTR_POSSIBLE_VALUE = { 'failonerror' : ['true', 'false'],
                            'ssl' : ['true','false'] } 

    def __init__(self, **kwargs):
        pyant.Task.__init__(self, **kwargs)

    def _checkadd(self):
        if 'To' in self._append_obj.keys():
            for i in self._append_obj['To']:
                self.tolist = i.address 
            if '_tolist' not in self._kwargs and len(self.tolist):
                self._kwargs.append('_tolist')
        if 'tolist' not in dir(self):
                raise AttributeError("%s does not attribute %s" % (self.__class__.__name__, 'tolist'))
        if 'Cc' in self._append_obj.keys():
            for i in self._append_obj['Cc']:
                self.cclist = i.address if not len(self.cclist) else self.cclist + ';' + i.value
        if 'Bcc' in self._append_obj.keys():
            for i in self._append_obj['Bcc']:
                self.bcclist = i.address if not len(self.bcclist) else self.bcclist + ';' + i.value
        if 'Replyto' in self._append_obj.keys():
            for i in self._append_obj['Replyto']:
                self.replyto = i.address if not len(self.replyto) else self.replyto + ';' + i.value
        if 'Message' in self._append_obj.keys():
            for i in self._append_obj['Message']:
                self.message = i.value if not len(self.message) else self.message + '\n' + i.value
        if 'Attachments' in self._append_obj.keys():
            for i in self._append_obj['Attachments']:
                self.files = i.value if not len(self.files) else self.files + ';' + i.value


    def _run(self):
        try:
            msg = MIMEMultipart()
            msg['Subject'] = self._subject
            msg['From'] = self._from
            msg['To'] = self._tolist
            msg['Cc'] = self._cclist
            msg['Bcc'] = self._bcclist
            msg['Reply-to'] = self._replyto
            if len(self._messagefile):
                body = open(os.path.join(self.project.basedir, self._messagefile), 'r').read() 
            else:
                body = self._message
            part = MIMEText(body, self._messagemimetype)
            msg.attach(part)
            for f in self._files.split(';'):
                part = MIMEBase('application', "octet-stream")
                part.set_payload( open(os.path.join(self.project.basedir,f),"rb").read() )
                Encoders.encode_base64(part)
                part.add_header('Content-Disposition', 'attachment; filename="%s"' % os.path.basename(f))
                msg.attach(part)
            if self._ssl == 'true':
                smtp = smtplib.SMTP_SSL(self._mailhost, self._mailport)
            else:    
                smtp = smtplib.SMTP(self._mailhost, self._mailport)
            if len(self._password):
                smtp.login(self._user, self._password)
            smtp.sendmail(self._from, self._tolist, msg.as_string())
            smtp.close()
            self.log('INFO','send mail %s to %s' % (self._subject, self._tolist))
        except Exception as e:
            if self._failonerror == 'false':
                self.log('DEBUG','error on send mail')
            else:
                raise e

pyant.ANTCLASS['mail'] = Mail


