#!/usr/bin/env python
# -*- coding: utf-8 -*-
 
"""
    Module pyant
    ant by python
"""
import os, sys
import os.path
import pyant
from mytools.util import *

class SqlTask(pyant.Task):
    """
    execute of SQL statements via module to a database
     
    name of element : **sql**
    
    ==================== ============================================= ========
    Attribute            Information                                   Required
    ==================== ============================================= ========
    module               module use for connexion                      Yes
    query                query for connexion
    property             insert into property vlue of return query
    path                 add path to pythonpath
    failonerror          stop if failure default true 
    ==================== ============================================= ========
    
    You can use element param for add parameter of connexion
    """
   
    ATTR_REQUIRED = ['module']
    ATTR_DEFAULT = {'query' : '',
                     'failonerror' : 'true',
                     'property' : '',
                     'path' : ''}
    ATTR_POSSIBLE_VALUE = {'failonerror' : ['true','false'] } 

    def __init__(self, **kwargs):
        pyant.Task.__init__(self, **kwargs)
        self.param = {}

    def _checkadd(self):
        if 'Param' in self._append_obj.keys():
            for i in self._append_obj['Param']:
                self.param[i.name] = i.get() 

    def __setattr__(self, item, value):
        if item == 'cdata':
            item = 'query'
        pyant.Elt.__setattr__(self, item, value)

    def _run(self):
        try:
            if len(self._path):
                d = os.path.join(self.project.basedir, self._path)
                if d not in sys.path:
                    sys.path.append(d)
                    self.log('DEBUG','append path %s' % d)
            mod = __import__(self._module, globals(), locals())
            cnxn = mod.connect(**self.param)
            cursor = cnxn.cursor()
            cursor.execute(self._query)
            if len(self._property):
                row = cursor.fetchone()
                self.parent.append_obj(pyant.Property(name=self._property, value=row[0]))
                self.log('DEBUG','add property %s with value %s' % (self._property, row[0]))
            cnxn.commit()
            cnxn.close()
            self.log('INFO','execute query sql')
        except Exception as e:
            if self._failonerror == 'false':
                self.log('DEBUG','error on send mail')
            else:
                raise e

pyant.ANTCLASS['sql'] = SqlTask


