#!/usr/bin/env python
# -*- coding: utf-8 -*-
 
"""
    Module pyant
    ant by python
"""
import os, sys, stat
import os.path
import shutil
import glob
import zipfile
import tarfile
import gzip
import pyant
from mytools.util import *


def addDirToZip(zipHandle, path, basePath=""):
    basePath = basePath.rstrip("\\/") + ""
    basePath = basePath.rstrip("\\/")
    for root, dirs, files in os.walk(path):
        for file in files:
            filePath = os.path.join(root, file)
            inZipPath = filePath.replace(basePath, "", 1).lstrip("\\/")
            zipHandle.write(filePath, inZipPath)

def addDirToTar(tarHandle, path, basePath=""):
    basePath = basePath.rstrip("\\/") + ""
    basePath = basePath.rstrip("\\/")
    for root, dirs, files in os.walk(path):
        for file in files:
            filePath = os.path.join(root, file)
            inZipPath = filePath.replace(basePath, "", 1).lstrip("\\/")
            tarHandle.add(filePath, inZipPath)



class Zip(pyant.Task):
    """
    Create a zipfile
    
    name of element : **zip**

    ==================== ============================================= ========
    Attribute            Information                                   Required
    ==================== ============================================= ========
    destfile             the zip-file to create                        Yes
    basedir              the directory from which to zip the files     Yes
    update               update a zip file if it exists default false 
    ==================== ============================================= ========
    """
    
    ATTR_REQUIRED = ['dir', 'destfile']
    ATTR_DEFAULT = { 'update' : 'false' }
    ATTR_POSSIBLE_VALUE = { 'update' : ['true','false'] } 

    def __init__(self, **kwargs):
        pyant.Task.__init__(self, **kwargs)

    def __setattr__(self, item, value):
        if item == 'basedir':
            item = 'dir'
            self._kwargs.append('_dir')
        pyant.Task.__setattr__(self, item, value)

    def _run(self):
        d = os.path.join(self.project.basedir, self._dir)
        zipf =  os.path.join(self.project.basedir, self._destfile)
        if os.path.exists(zipf) and self._update == 'false':
            os.remove(zipf)
        with zipfile.ZipFile(zipf, 'a') as myzip:
            if os.path.isdir(d):
                addDirToZip(myzip, d, os.path.dirname(d))
            else:
                myzip.write(d, os.path.basename(d))
        self.log('INFO','Created zip file : %s' % os.path.abspath(d))     

pyant.ANTCLASS['zip'] = Zip

class Unzip(pyant.Task):
    """
    unzip file zip in directory
    
    name of element : **unzip**

    ==================== ============================================= ========
    Attribute            Information                                   Required
    ==================== ============================================= ========
    src                  the zip-file                                  Yes
    dest                 the directory for extract                     Yes
    ==================== ============================================= ========
    """
    
    ATTR_REQUIRED = ['src', 'dest']
    ATTR_DEFAULT = {}
    ATTR_POSSIBLE_VALUE = {} 

    def __init__(self, **kwargs):
        pyant.Task.__init__(self, **kwargs)

    def _run(self):
        dest = os.path.join(self.project.basedir, self._dest)
        zipf =  os.path.join(self.project.basedir, self._src)
        with zipfile.ZipFile(zipf, 'r') as myzip:
            myzip.extractall(dest)
        self.log('INFO','Extract zip file %s in %s' % (self._src, os.path.abspath(dest)))     

pyant.ANTCLASS['unzip'] = Unzip

class Tar(pyant.Task):
    """
    Create a tar archive
    
    name of element : **tar**

    ==================== ============================================= ========
    Attribute            Information                                   Required
    ==================== ============================================= ========
    destfile             the zip-file to create                        Yes
    basedir              the directory from which to zip the files     Yes
    update               update a zip file if it exists default false 
    ==================== ============================================= ========
    """
    
    ATTR_REQUIRED = ['dir', 'destfile']
    ATTR_DEFAULT = { 'update' : 'false' }
    ATTR_POSSIBLE_VALUE = { 'update' : ['true','false'] } 

    def __init__(self, **kwargs):
        pyant.Task.__init__(self, **kwargs)

    def __setattr__(self, item, value):
        if item == 'basedir':
            item = 'dir'
            self._kwargs.append('_dir')
        pyant.Task.__setattr__(self, item, value)


    def _run(self):
        d = os.path.join(self.project.basedir, self._dir)
        tarf =  os.path.join(self.project.basedir, self._destfile)
        if os.path.exists(tarf) and self._update == 'false':
            os.remove(tarf)
        with tarfile.TarFile(tarf, 'a') as mytar:
            if os.path.isdir(d):
                addDirToTar(mytar, d, os.path.dirname(d))
            else:
                mytar.add(d, os.path.basename(d))
        self.log('INFO','Created tar file : %s' % os.path.abspath(d))     

pyant.ANTCLASS['tar'] = Tar

class Untar(pyant.Task):
    """
    extract archive tar
    
    name of element : **untar**

    ==================== ============================================= ========
    Attribute            Information                                   Required
    ==================== ============================================= ========
    src                  the zip-file                                  Yes
    dest                 the directory for extract                     Yes
    ==================== ============================================= ========
    """
    
    ATTR_REQUIRED = ['src', 'dest']
    ATTR_DEFAULT = {}
    ATTR_POSSIBLE_VALUE = {} 

    def __init__(self, **kwargs):
        pyant.Task.__init__(self, **kwargs)

    def _run(self):
        dest = os.path.join(self.project.basedir, self._dest)
        tarf =  os.path.join(self.project.basedir, self._src)
        with tarfile.TarFile(tarf, 'r') as mytar:
            mytar.extractall(dest)
        self.log('INFO','Extract tar file %s in %s' % (self._src, os.path.abspath(dest)))     

pyant.ANTCLASS['untar'] = Untar

class Gzip(pyant.Task):
    """
    Create a gzip archive
    
    name of element : **gzip**

    ==================== ============================================= ========
    Attribute            Information                                   Required
    ==================== ============================================= ========
    destfile             the zip-file to create                        Yes
    src                  the file to gzip                              Yes
    ==================== ============================================= ========
    """
    
    ATTR_REQUIRED = ['src', 'destfile']
    ATTR_DEFAULT = {}
    ATTR_POSSIBLE_VALUE = {} 

    def __init__(self, **kwargs):
        pyant.Task.__init__(self, **kwargs)

    def _run(self):
        d = os.path.join(self.project.basedir, self._src)
        gzipf =  os.path.join(self.project.basedir, self._destfile)
        f_in = open(d, 'rb')
        f_out = gzip.open(gzipf, 'wb')
        f_out.writelines(f_in)
        f_out.close()
        f_in.close()
        self.log('INFO','Created gzip file : %s' % os.path.abspath(d))     

pyant.ANTCLASS['gzip'] = Gzip

class Gunzip(pyant.Task):
    """
    extract archive gzip
    
    name of element : **gunzip**

    ==================== ============================================= ========
    Attribute            Information                                   Required
    ==================== ============================================= ========
    src                  the zip-file                                  Yes
    dest                 the directory for extract                     
    ==================== ============================================= ========
    """
    
    ATTR_REQUIRED = ['src']
    ATTR_DEFAULT = {}
    ATTR_POSSIBLE_VALUE = {} 

    def __init__(self, **kwargs):
        pyant.Task.__init__(self, **kwargs)

    def _checkadd(self):
        if len(self._dest) == 0:
            self.dest = '.'.join(self._src.split('.')[:-1])

    def _run(self):
        dest = os.path.join(self.project.basedir, self._dest)
        gzipf =  os.path.join(self.project.basedir, self._src)
        inF = gzip.open(gzipf, 'rb')
        outF = open(dest, 'wb')
        outF.write( inF.read() )
        inF.close()
        outF.close()
        self.log('INFO','Extract gzip file %s in %s' % (self._src, os.path.abspath(dest)))     

pyant.ANTCLASS['gunzip'] = Gunzip


